#--*-coding:utf-8-*--

class KillDeathBusiness < DiceBot
  
  def initialize
    super
    @sendMode = 2
    @sortType = 1
    @d66Type = 2
  end
  
  def prefixs
    [
     'HST',
     'ST[1-2]?',
     'DWT', 'DeathWT',
     'RWT', 'RevengeWT',
     'VWT', 'VictoryWT',
     'PWT', 'PossesionWT',
     'CWT', 'ControlWT',
     'FWT', 'FlourishWT',
     'IWT', 'IntensifyWT',
     'HWT', 'HealthWT',
     'SaWT', 'SafetyWT',
     'LWT', 'LongevityWT',
     'EWT', 'ExistWT',
     'NAME[1-3]?', 'NAME[1-3]?',
     'OSPT', 'OccultSPT',
     'FSPT', 'FamilySPT',
	 'LoSPT', 'LoveSPT',
     'JSPT', 'JusticeSPT',
     'TSPT', 'TrainingSPT',
     'BSPT', 'BeamSPT',
     'CMT',
     'EST', 'sErviceST',
     'SOUL',
     'STGT',
     'HSAT[1-2]?',
	 'EXT[1-4]?',
     'SKLT',
	 'SKLJ',
	 'ERT',
	 'WKT',
	 'PCDT',
	 'OHT',
	 'PCT1',
	 'PCT2',
	 'PCT3',
	 'PCT4',
	 'PCT5',
	 'PCT6',
	 'PCT7',
	 'JD.*',
	]
  end
  
  def gameName
    'キルデスビジネス'
  end
  
  def gameType
    "KillDeathBusiness"
  end
  
  def getHelpMessage
    return <<INFO_MESSAGE_TEXT
・判定
　JDx or JDx+y or JDx-y or JDx,z or JDx+y,z JDx-y,z
　（x＝難易度、y＝補正、z＝ファンブル率(リスク)）
・履歴表 (HST、HSTx) xに数字(1,2)で表を個別ロール
・願い事表 (-WT)
　死(DWT)、復讐(RWT)、勝利(VWT)、獲得(PWT)、支配(CWT)、繁栄(FWT)
　強化(IWT)、健康(HWT)、安全(SAWT)、長寿(LWT)、生(EWT)
・万能命名表 (NAME) xに数字(1,2,3)で表を個別ロール
・サブプロット表 (-SPT)
　オカルト(OSPT)、家族(FSPT)、恋愛(LOSPT)、正義(JSPT)、修行(TSPT)
　笑い(BSPT)、意地悪(MASPT)、恨み(UMSPT)、人気(POSPT)、仕切り(PASPT)
　金儲け(MOSPT)、対悪魔(ANSPT)
・シーン表 (ST)、サービスシーン表 (EST)
・CM表 (CMT)
・蘇生副作用表 (ERT)
・一週間表（WKT)
・ソウル放出表 (SOUL)
・汎用演出表 (STGT)
・ヘルスタイリスト罵倒表 (HSAT、HSATx) xに数字(1,2)で表を個別ロール
・指定特技ランダム決定表 (SKLT)、指定特技分野ランダム決定表 (SKLJ)
・エキストラ表 (EXT、EXTx) xに数字(1,2,3,4)で表を個別ロール
・製作委員決定表　PCDT/実際どうだったのか表　OHT
・タスク表　ヘルライオン　PCT1/ヘルクロウ　PCT2/ヘルスネーク　PCT3/
　ヘルドラゴン　PCT4/ヘルフライ　PCT5/ヘルゴート　PCT6/ヘルベア　PCT7
・D66ダイスあり
INFO_MESSAGE_TEXT
  end
  
  
  # ゲーム別成功度判定(2D6)
  def check_2D6(total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max)
    
    debug("total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max", total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max)
    
    return '' unless(signOfInequality == ">=")
    
    output = 
      if(dice_n <= 2)
        " ＞ ファンブル(判定失敗。【視聴率】が20％減少)"
      elsif(dice_n >= 12)
        " ＞ スペシャル(判定成功。【視聴率】が10％増加)"
      elsif(total_n >= diff)
        " ＞ 成功"
      else
        " ＞ 失敗"
      end
    
    return output
  end
  
  
  def rollDiceCommand(command)
    
    debug("rollDiceCommand command", command)
    
    #判定チェックは先に処理
    case command
    when @@judogeDiceReg
      result = judgeDice(command)
      text = "判定#{result}"
      return text
    end
    
    #判定以外なら表コマンドの処理に
    return rollTableCommand(command)
  end
  
  
  @@judogeDiceReg = /(^|\s)JD(\d+)([\+\-]\d+)?(,(\d+))?($|\s)/i
  
  def judgeDice(command)
    
    unless(@@judogeDiceReg === command)
      return '1'
    end
    
    target = $2.to_i
    modify = $3.to_i
    fumble = $5.to_i
    
    result = ""
    
    if(target > 12 )
      result  += "【#{command}】 ＞ 難易度が12以上はスペシャルのみ成功。\n"
      target = 12
    end
    
    if(target < 5 )
      result  += "【#{command}】 ＞ 難易度の最低は5。\n"
      target = 5
    end
    
    if( fumble < 2 )
      fumble = 2
    elsif(fumble > 11 )
      result  += "【#{command}】 ＞ スペシャルを出すと必ず成功なので、ファンブル率は11とする。\n"
      fumble = 11
    end
    
    
    number, diceText, = roll(2, 6)
    
    result += "【難易度#{target}、補正#{modify}、ファンブル率#{fumble}】 ＞ 出目(#{diceText}) ＞ "
    
    if(number == 2 )
      result += "出目が2なのでファンブル！(判定失敗。【視聴率】が20％減少)"
    elsif(number == 12)
      result += "出目が12なのでスペシャル！(判定成功。【視聴率】が10％増加)"
    elsif(number <= fumble)
      result += "出目がファンブル率以下なのでファンブル！(判定失敗。【視聴率】が20％減少)"
    else
       number += modify
       if(number < target)
         result += "達成値#{number}、難易度未満なので判定失敗！"
       else
         result += "達成値#{number}、難易度以上なので判定成功！"
       end
    end
    
        
    return result
  end
  
  
  
  def rollTableCommand(command)
    
    tableName = ""
    result = ""
    
	case command
      
    when "HST"
      tableName, result, number = getHistoryTableResult
      
    when /^ST(\d)?$/
	  #シーン表
      type = $1.to_i
      
	  tableName, result, number = getSceneTableResult(type)
      
    when /^.+WT$/i
	  #願い事表
      tableName, result, number = getWishTableResult(command)
      
    when /^NAME(\d)?$/
	  #万能命名表
      type = $1.to_i
      tableName, result, number = getNameTableResult(type)
      
    when /^.+SPT$/i
	  #サブプロット表
      tableName, result, number = getSubprotTableResult(command)
      
    when "CMT"
	  #CM表
      tableName, result, number = getCmTableResult()
      
    when "ERT"
	  #副作用蘇生表
      tableName, result, number = getErTableResult()
      
    when "WKT"
	  #一週間表
      tableName, result, number = getWKTableResult() 
            
    when /^EST$/i, /^sErviceST$/i
      tableName, result, number = getServiceSceneTableResult()
      
    when "SOUL"
      tableName, result, number = getSoulTableResult()
	  
    when "STGT"
      tableName, result, number = getSceneTelopGenericTableResult()
	  
    when /^HSAT(\d)?$/
      type = $1.to_i
      tableName, result, number = getHairStylistAbuseTableResult(type)
	  
    when /^EXT(\d)?$/
      type = $1.to_i
      tableName, result, number = getExtraTableResult(type)
	  
    when /^SKL(T|J)$/
	  type = $1
      tableName, result, number = getSkillTableResult(type)
      
    when "PCDT"
	  #製作委員決定表
      tableName, result, number = getpcTableResult()
    when "OHT"
	  #実際どうだったのか表
      tableName, result, number = getohTableResult()
    when "PCT1"
	  #ヘルライオンタスク表
      tableName, result, number = getplTableResult()
    when "PCT2"
	  #ヘルクロウタスク表
      tableName, result, number = getprTableResult()
    when "PCT3"
	  #ヘルスネークタスク表
      tableName, result, number = getpnTableResult()
    when "PCT4"
	  #ヘルドラゴンタスク表
      tableName, result, number = getpdTableResult()
    when "PCT5"
	  #ヘルフライタスク表
      tableName, result, number = getpfTableResult()
    when "PCT6"
	  #ヘルゴートタスク表
      tableName, result, number = getpgTableResult()
    when "PCT7"
	  #ヘルベアタスク表
      tableName, result, number = getpbTableResult()

    else
      debug("rollDiceCommand commandNOT matched -> command:", command)
      return ""
	end
    
    if( result.empty? )
      return ""
    end
	
	text = "#{tableName}(#{number})：#{result}"
	return text
  end
  
  def getHistoryTableResult
    tableName = "履歴表"
    
    table = [
             '魔術
あなたは悪魔との接触を目指して研磨を積んでいたオカルティスト、または恐怖の探索者である。たいへんな苦労の末、ようやく現れた悪魔は、願いと引き替えに番組に出ろ、などと言い出した。あなたは少々あっけにとられながらも、そもそも悪魔を探していた理由である願い事を叶えるため、彼らの主催する番組に出演する事を承諾した。',
             '天才
あなたはある分野での天才である。なんの天才であるのかは、キャラクター作成を進めるうちに思いついたら決めてもいいし、まったく決めなくてもかまわない。とにかく、天才が番組に登場するとキャラが立って面白くなると考えた悪魔は、あなたの目の前に番組の出演交渉のために現れた。あなたはその話に乗り、魂の回収人になることに決めた。',
             '狂気
あなたは頭がおかしい。そのため、周囲の社会とうまくかみ合わない人生を送っていた。悪魔はそこに出現し、社会をあなたに合わせて変えてしまうことが可能だと言った。ただし人を殺すゲーム番組で優勝しなければならないらしい。しかし、あなたは頭がおかしいので、人を殺すところは気にもならなかった。とにかく勝てば幸せになれるのだ。がんばろう。',
             '絶望
あなたはなんらかの失敗によって失意のどん底にあった。合法、あるいは違法な薬物に溺れ、生活はどんどん荒み、そろそろ死んでしまおうかと考えているところに悪魔が現れた。そいつの言うことには、なんかリアリティ番組で勝てば、この状況から抜け出せるらしい。もはや失うものなどないあなたは、その話に飛びつき、番組に出演することにした。',
             '先達
あなたは過去『キルデスビジネス』の勝利者と知り合いである。急に理由もなく大きな成功をおさめた知り合いに事情を聞くと、こっそりと悪魔が主催する殺人ゲーム番組のことを教えてくれた。あなたは願い事を叶えてもらうために、その知り合いを通して悪魔に推薦してもらい、番組に参加することになった。知り合いは、きっと番組を見ているだろう。',
             '友人
あなたは平穏な人生を送っていたが、あるとき目の前に悪魔が現れる。そいつは番組出演のエントリーシートを持っていて、そこにはあなたの名前が書かれていた。友人が勝手にあなたの名前で番組にエントリーシートを送っていたのだ！　あなたはそのまま拉致され、むりやり願い事を決めさせられ、悪趣味な番組で暴力を撒き散らす事になってしまった。',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end
  
  
  def getSceneTableResult(type)
    debug("getSceneTableResult type", type)
    
    tableName = "シーン表"
    
    sceneTable1 = [
                   [11, 'あなたが根城にしていた'],
                   [12, 'あなたの写真で壁が埋め尽くされた'],
                   [13, 'アナログテレビが山と積まれた'],
                   [14, 'ゴミで埋め尽くされた'],
                   [15, 'ヘル'],
                   [16, '押し寄せてくる砂に埋もれた'],
                   [22, '価値あるもので豪華に飾られた'],
                   [23, '監視機器で電子の要塞と化した'],
                   [24, '奇妙なオブジェだらけの'],
                   [25, '巨大な穴のふちに危うく引っかかった'],
                   [26, '極彩色の壁画に彩られた'],
                   [33, '消えかけて点滅する明かりに照らされた'],
                   [34, '青白く何もかもが凍りついた'],
                   [35, '赤い血と鉄のにおいに満ちた'],
                   [36, '太陽が強烈な光を投げかける'],
                   [44, '鳥が無数に集まってきている'],
                   [45, '燃え上がる炎に照らされた'],
                   [46, '白い霧で少し先すら見えない'],
                   [55, '腐った死体が折り重なる'],
                   [56, '有毒な汚水がたっぷり詰まった'],
                   [66, '略奪を受けて空っぽになった'],
                  ]
    sceneTable2 = [
                   [11, 'オフィス'],
                   [12, '祭壇'],
                   [13, 'スタジオ'],
                   [14, '倉庫'],
                   [15, 'バス停'],
                   [16, '牢獄'],
                   [22, '暗闇'],
                   [23, '駅'],
                   [24, '屋上'],
                   [25, '下水道'],
                   [26, '学校'],
                   [33, '研究室'],
                   [34, '車内'],
                   [35, '森'],
                   [36, '庭園'],
                   [44, '店'],
                   [45, '農園'],
                   [46, '廃墟'],
                   [55, '墓地'],
                   [56, '民家'],
                   [66, '野原'],
                  ]
    
    result = ''
    number = 0
    
    case type
    when 1
      result, number = get_table_by_d66_swap(sceneTable1)
    when 2
      result, number = get_table_by_d66_swap(sceneTable2)
    else
      result1, num1 = get_table_by_d66_swap(sceneTable1)
      result2, num2 = get_table_by_d66_swap(sceneTable2)
      result = "#{result1}#{result2}"
      number = "#{num1},#{num2}"
    end
    
    return tableName, result, number
  end
  
  
  def getWishTableResult(command)
    debug("getWishTableResult command", command)
    
    tableName = ''
    table = []
    
    case command
    when /^DWT$/i, /^DeathWT$/i
      tableName = "願い事表「死」"
      table = [
               '今の美しさを保ったまま死にたい。',
               'おもしろい死にかたをしてみたい。',
               '自殺する勇気がないので死にたい。',
               '死んだらどうなるのか見てみたい。',
               'とても嫌なことがあったので死にたい。',
               '不老不死に飽きたので死にたい。',
              ]
    when /^RWT$/i, /^RevengeWT$/i
      tableName = "願い事表「復讐」"
      table = [
               '家族を殺した犯人に復讐したい。',
               '自分より幸せな隣人に復讐したい。',
               '自分を裏切った友人に復讐したい。',
               '自分を捨てた恋人に復讐したい。',
               '大事なものを盗んだ犯人に復讐したい。',
               '手ひどく負けたライバルに復讐したい。',
              ]
    when /^VWT$/i, /^VictoryWT$/i
      tableName = "願い事表「勝利」"
      table = [
               'かつて負けたライバルに勝利したい。',
               '倒したい有名人(故人)がいる。',
               '倒したい有名人がいる。',
               '特定の業界で一番の実力者になりたい。',
               '特定のゲームで最強のプレイヤーになりたい。',
               '特定のテストで一位になりたい。',
              ]
    when /^PWT$/i, /^PossesionWT$/i
      tableName = "願い事表「獲得」"
      table = [
               '欲しいアイテムがある。',
               '欲しい生き物がいる。',
               '欲しい関係がある。',
               '欲しい親族がいる。',
               '欲しい知識がある。',
               '欲しい不動産がある。',
              ]
    when /^CWT$/i, /^ControlWT$/i
      tableName = "願い事表「支配」"
      table = [
               '支配したい人物集団がいる。',
               '支配したい地域がある。',
               '世界を征服したい。',
               '選挙で勝ちたい。',
               '這いつくばらせたい人がいる。',
               'わがままな縁者を思い通りにしたい。',
              ]
    when /^FWT$/i, /^FlourishWT$/i
      tableName = "願い事表「繁栄」"
      table = [
               '落ちぶれた一族を復権させたい。',
               '地元経済を活性化させたい。',
               '趣味のジャンルに人を集めたい。',
               '絶滅した生き物を復活させたい。',
               '動物や植物を絶滅から救いたい。',
               'とにかくたくさんお金が欲しい。',
              ]
    when /^IWT$/i, /^IntensifyWT$/i
      tableName = "願い事表「強化」"
      table = [
               '世界最強になりたい。',
               '世界で最も美しくなりたい。',
               '世界で最も賢くなりたい。',
               '世界で最も幸せになりたい。',
               '世界で最もタフになりたい。',
               '世界で最も速くなりたい。',
              ]
    when /^HWT$/i, /^HealthWT$/i
      tableName = "願い事表「健康」"
      table = [
               '完成させたい薬物がある。',
               '絶滅させたい病原菌がある。',
               '難病から回復させたい縁者がいる。',
               '難病から回復したい。',
               '病気で死なせてしまった人を復活させたい。',
               '病気にならない体が欲しい。',
              ]
    when /^SaWT$/i, /^SafetyWT$/i
      tableName = "願い事表「安全」"
      table = [
               '穏やかな生活を送りたい。',
               '穏やかに暮らして欲しい人がいる。',
               '終わらせたい争いがある。',
               '事故で死なせてしまった人を復活させたい。',
               '事故に遭わない能力が欲しい。',
               '迫り来る世界の危機を回避したい。',
              ]
    when /^LWT$/i, /^LongevityWT$/i
      tableName = "願い事表「長寿」"
      table = [
               '今の美しさを保ちたい。',
               '今の強さを保ちたい。',
               '子供が一人前になるまで死ねない。',
               '不老の体が欲しい。',
               'ボケたくない。',
               '孫の顔を見てみたい。',
              ]
    when /^EWT$/i, /^ExistWT$/i
      tableName = "願い事表「生」"
      table = [
               '生きている実感が欲しい。',
               '今とは違う人生をやり直したい。',
               '死んでしまったので復活したい。',
               '死んでしまった人を復活させたい。',
               '生レバー復活しないかな。',
               '理想の人生を手に入れたい。',
              ]
    end
    
    result, number = get_table_by_1d6(table)
    debug("getWishTableResult result", result)
    
    return tableName, result, number
  end
  
  
  def getNameTableResult(type)
    tableName = "万能命名表"
    
    nameTable1 = [
                  [11, 'アナログ'],
                  [12, 'イージー'],
                  [13, 'エターナル'],
                  [14, 'エレクトロ'],
                  [15, 'オート'],
                  [16, 'キング'],
                  [22, 'クール'],
                  [23, 'グリム'],
                  [24, 'グレート'],
                  [25, 'スーパー'],
                  [26, 'スプーク'],
                  [33, 'スマート'],
                  [34, 'ドレッド'],
                  [35, 'ハイテク'],
                  [36, 'ハンターズ'],
                  [44, 'ブラック'],
                  [45, 'ヘル'],
                  [46, 'マジック'],
                  [55, 'メカニカル'],
                  [56, 'ラヴ'],
                  [66, 'レッド'],
                 ]
    nameTable2 = [
                  [11, 'X'],
                  [12, 'アビス'],
                  [13, 'グレイブ'],
                  [14, 'ゴールド'],
                  [15, 'ゴシック'],
                  [16, 'シノビ'],
                  [22, 'スペシャル'],
                  [23, 'デーモン'],
                  [24, 'デジタル'],
                  [25, 'デス'],
                  [26, 'デビル'],
                  [33, 'ナイトメア'],
                  [34, 'バロック'],
                  [35, 'ブラッド'],
                  [36, 'ブレイク'],
                  [44, 'ブレイズ'],
                  [45, 'フロスト'],
                  [46, 'メガ'],
                  [55, 'メカナイズド'],
                  [56, 'リンボ'],
                  [66, 'ロボ'],
                 ]
    nameTable3 = [
                  [11, 'アーム'],
                  [12, 'アイ'],
                  [13, 'イヤー'],
                  [14, 'ウィング'],
                  [15, 'エンジン'],
                  [16, 'カード'],
                  [22, 'ガン'],
                  [23, 'ギア'],
                  [24, 'キャノン'],
                  [25, 'サーキット'],
                  [26, 'スキン'],
                  [33, 'ソード'],
                  [34, 'タービン'],
                  [35, 'ハート'],
                  [36, 'ハンド'],
                  [44, 'ビーム'],
                  [45, 'プレート'],
                  [46, 'ヘア'],
                  [55, 'メカニズム'],
                  [56, 'モーター'],
                  [66, 'レッグ'],
                 ]
    
    result = ''
    number = 0
    
    case type
    when 1
      result, number = get_table_by_d66_swap(nameTable1)
    when 2
      result, number = get_table_by_d66_swap(nameTable2)
    when 3
      result, number = get_table_by_d66_swap(nameTable3)
    else
      result1, num1 = get_table_by_d66_swap(nameTable1)
      result2, num2 = get_table_by_d66_swap(nameTable2)
      result3, num3 = get_table_by_d66_swap(nameTable3)
      result = "#{result1}#{result2}#{result3}"
      number = "#{num1},#{num2},#{num3}"
    end
    
    return tableName, result, number
  end
  
  
  def getSubprotTableResult(command)
    tableName = ''
    table = []
    
    case command
    when /^OSPT$/i, /^OccultSPT$/i
      tableName = "サブプロット表「オカルト」"
      table = [
               '『死の探求』
回収人から一人を選び、相手が「職業」で修得している特技を指定特技として判定を行う。判定に成功すると、番組による頻繁な死が相手をどう変えていくのか観察したくなる。相手への関係を属性「研究対象」で獲得する。【深度】は１になる。
クリア条件：相手が、あなたの登場しているシーンで二回死亡する。死亡によって相手がどう変わったのか、勝手なことをつらつらと語ってもいいし、なにも語らずにわかったような顔をしていても構わない。
報酬：【視聴率】10%増加、400【ソウル】',
               '『来訪者』
キャラクターから一人を選び、相手が「衣装」で修得している特技を指定特技として判定を行う。判定に成功すると、相手が着ているものは巧妙な偽装であり、その正体が宇宙人であることに気付く。相手への関係を属性「宇宙人」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を２以上にする。すると、相手は自分の本当の姿を教えてくれる。本当に宇宙人なのかどうかは相手次第で、別にどちらでもかまわない。適当に喜んだりがっかりしておくこと。
報酬：【視聴率】10%増加、200【ソウル】',
               '『記憶喪失』
キャラクターから一人を選び、相手が「願望」で修得している特技を指定特技として判定を行う。判定に成功すると、その瞬間に、相手が何かを知っているという確信だけを残し、あとのすべての個人的な記憶を失う。相手への関係を、属性「知人」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を３にする。すると、相手はあなたがどんな人物だったのか思い出させてくれる。直接思い出しても構わないし、なにげない仕草が思い出すきっかけになるのかもしれない。
報酬：【視聴率】10%増加、相手が回収人なら400【ソウル】、相手がNPCなら600【ソウル】',
               '『秘密結社』
キャラクターから一人を選び、相手が「動作」で修得している特技を指定特技として判定を行う。判定に成功すると、相手がなにげなく行っていた動作が、あなたの所属する秘密結社のメンバーであることを示すサインだと見抜く。相手への関係を、属性「メンバー」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を２以上にする。相手は本当にメンバーなのかもしれないし、あなたの勘違いかもしれない。どちらにせよ、探りを入れながら近づくことで、相手のことがよくわかった。
報酬：【視聴率】10%増加、200【ソウル】',
               '『殺人鬼』
キャラクターから一人を選び、相手が「情動」で修得している特技を指定特技として判定を行う。判定に成功すると、相手が有名な連続殺人犯の犯人であると確信する。顔や性別は変えたのかもしれない。相手への関係を、属性「殺人鬼」で獲得する。【深度】は１になる。
クリア条件：連続殺人鬼を許せないなら、相手を戦闘で一度倒す。または、連続殺人鬼のファンなら、相手と一緒に「サービス」シーンを成功させる。どちらにせよ、相手が本当に連続殺人をやったのかははっきりしない。
報酬：【視聴率】10%増加、クリア条件達成時の相手への関係【深度】× 200【ソウル】',
               '『魔法使い』
キャラクターから一人を選び、相手が「小道具」で修得している特技を指定特技として判定を行う。判定に成功すると、相手はもともと本当に魔法が使える人間であり、持っている道具は高名なマジックアイテムだと確信する。相手への関係を、属性「魔法使い」で獲得する。【深度】は１となる。
クリア条件：相手への関係の【深度】を２以上にし、かつ相手からあなたへの関係の【深度】を1以上にする。根負けした相手は魔法を教えてくれて、あなたは満足する。ちなみに、この魔法は効果があいまいでハッキリしないものを適当に設定する。
報酬：【視聴率】10%増加、あなたと相手の両方に300【ソウル】',
              ]
    when /^FSPT$/i, /^FamilySPT$/i
      tableName = "サブプロット表「家族」"
      table = [
               '『説得』
キャラクターから一人選び、相手が「小道具」で習得している特技を指定特技として判定を行う。判定に成功すると、相手が持っているのはあなたの贈りものであり、旧友が悪趣味なショーに出ているのだとわかる。降りるように説得しなければ。相手への関係を、属性「旧友」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を３にする。すると、説得をやるだけやったあなたは満足する。相手は多分諦めない。または、選択された「旧友」が死亡するか（回収人なら）地獄に墜ちる。すると、なんだかあきらめがつく。
報酬：【視聴率】10%増加、400【ソウル】',
               '『生き別れ』
キャラクターから一人選び、相手が「動作」で修得している特技を指定特技として判定を行う。判定に成功すると、相手が家族特有の特徴を持っていることがわかり、かなり昔に生き別れた家族だと確信する。相手への関係を、属性「生き別れ」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を３にする。すると、相手はあなたのことを思い出す。まるで違う環境で暮らしてきたのかもしれないが、あなたたちのルーツは同じだ。続柄や生別したときの状況は好きに設定できるし、別に設定しなくてもかまわない。
報酬：【視聴率】10%増加、相手が回収人なら400【ソウル】相手がNPCなら600【ソウル】',
               '『家族自慢』
回収人ーから一人選び、相手が「職業」で修得している特技を指定特技として判定を行う。判定に成功すると、相手の職業を聞き出したうえで、同じ分野で相手より成功している家族の話を聞かせてやれる。相手への関係を、属性「自慢済み」で獲得する。【深度】は１になる。
クリア条件：三人のキャラクターへ【深度】１以上の関係を獲得し、すべての属性を「自慢済み」にする。すでに「自慢済み」の人は、死んでいたりいなくなっても大丈夫。かなり嫌がられそうだが、まあ仕方がない。
報酬：【視聴率】10%増加、400【ソウル】',
               '『捨て猫』
回収人から一人選び、相手が「感情」で修得している特技を指定特技として判定を行う。判定に成功すると、懐から仔猫が出てくる。これは相手が故あって捨てた猫だが、あなたはそれが許せない。相手への関係を、属性「猫の飼い主」で獲得する。【深度】は１になる。
クリア条件：相手に猫を飼わせるため、戦闘で一度倒す。すると、相手は猫を飼うことに同意する。猫の詳細なスペックを決めてもいいし、決めなくてもかまわない。あと、動物の種類も実は自由だ。
報酬：【視聴率】10%増加、クリア条件達成時の相手への関係の【深度】× 200【ソウル】',
               '『義理の家族』
回収人から一人選び、相手が「衣装」で修得している特技を指定特技として判定を行う。判定に成功すると、衣装の傾向から、相手が義理の家族であることが判明する。こんなところでなにをやっているんだ。相手への関係を、属性「義理の家族」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を２以上にする。すると、相手はなぜ番組に出ることになったのか、そのときどんな状況や気分だったのかを話してくれる。または、相手が地獄に墜ちると、縁が切れてクリアだ。
報酬：【視聴率】10%増加、200【ソウル】',
               '『孤独な人』
キャラクターから一人選び、相手が「願望」で修得している特技を指定特技として判定を行う。判定に成功すると、相手が孤独に耐えている寂しい人物であることを確信する。あなたが相手を救うのだ。相手への関係を、属性「孤独な人」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を２以上にする。すると、相手は打ち解ける……かもしれないし、なにも変わないかもしれない。ただ、あなたには、相手の孤独が少し癒やされたことがわかる。
報酬：【視聴率】10%増加、200【ソウル】',
              ]
    when /^LoSPT$/i, /^LoveSPT$/i
      tableName = "サブプロット表「恋愛」"
      table = [
               '『禁断の恋』
標的が「衣装」で修得している特技を指定特技として判定を行う。標的がすでに死亡している場合、サブプロット表を振りなおす。判定に成功すると、標的に一目惚れするか、標的が前から好きだった人であったことが判明する。急がないと殺されてしまう。相手への関係を、属性「好きな人」で獲得する。【深度】は１となる。
クリア条件：相手への関係の【深度】を３にする。すると、標的はあなたの思いに対してなんらかの答えを返す。結果がどうあれ、いい経験になるだろう。または、相手への関係の【深度】が２以上の状態で、相手が死亡してもいい。嘆こう。
報酬：【視聴率】10%増加、400【ソウル】',
               '『番組内デート』
回収人から一人を選び相手が「情動」で修得している特技を指定特技として判定を行う判定に成功すると、相手とあなたは番組開始前からつきあっている。収録のついでに仲を進展させてもいいかもしれない。相手への関係を、属性「恋人」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を２以上にし、相手からあなたの関係の【深度】も２以上にする。するとけっこういい感じに関係が発展する。番組終了時に視聴者にお知らせすることがあるかもしれない。
報酬：【視聴率】10%増加、あなたと相手の両方に400【ソウル】',
               '『悩み相談』
キャラクターから一人を選び相手が「職業」で修得している特技を指定特技として判定を行う。判定に成功すると、相手の職業的見地から恋愛に関するアドバイスをもらえるかもしれないと気づく。相手への関係を属性「相談役」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】は３にする。恋愛に関するアドバイスをもらった気になる。相手がその気で言ったかは関係ない。最後に関係の属性を『好きな人』に変えたりするとそれっぽい。
報酬：【視聴率】10%増加、相手が回収人なら400【ソウル】相手がNPCなら600【ソウル】',
               '『人気獲得』
キャラクターから一人を選び相手が「動作」で修得している特技を指定特技として判定を行う。判定に成功すると、相手とあなたが並ぶと非常に画面映えして視聴者に受けると確信する。誘ってサービスシーンを成功させればヘル視聴者に愛されるだろう。相手への関係を属性「相棒」で獲得する。【深度】は１になる。
クリア条件：相手と一緒にサービスシーンを成功させる。驚くべきことに、ヘル視聴者は本当にコンビが気に入ってしまう。ただ、もう一度同じことをやっても、ウケるかどうかはわからない。
報酬：【視聴率】10%増加、200【ソウル】',
               '『贈り物』
キャラクターから一人を選び相手が「願望」で修得している特技を指定特技として判定を行う。判定に成功すると、相手の望みがなんとなくわかる。欲しいものすらわかるので、あげたら好きになってもらえるかもしれない。なにが欲しいのか、相手に実際に尋ねる必要はない。相手への関係を属性「欲しがり」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を２以上にする。すると相手は貴方のプレゼントを受け取る。何を渡したのかは適当に決めてかまわないが、迷ったらヘルCM表をつかうこと。
報酬：【視聴率】10%増加、200【ソウル】',
               '『手料理』
キャラクターから一人を選び相手が「小道具」で修得している特技を指定特技として判定を行う。判定に成功すると、相手の持っているものがヒントになって、すごい料理を思いつく。もしかしたら材料にするのかもしれない。相手への関係を属性「毒見役」で獲得する。【深度】は１になる。
クリア条件：相手に戦闘を挑み、キラートリックで倒す。このとき、キラートリックの名前は思いついた料理の名前に変更される。困ったら万能命名表を使うこと。
報酬：【視聴率】10%増加、500【ソウル】',
              ]
    when /^JSPT$/i, /^JusticeSPT$/i
      tableName = "サブプロット表「正義」"
      table = [
               '『殺せない標的』
標的が「情動」で修得している特技を指定特技として判定を行う。判定に成功すると、相手があなたの正義に照らし合わせて申し分のない人物である事が分かる(もしくは、実は非常に善良な人物であったことがわかる)。これを殺させるわけにはいかない。相手への関係を、属性「護衛対象」で獲得する。【深度】は１になる。
クリア条件：結果フェイズまで相手を生き残らせる。なぜ味方をしたのか、なぜ味方をするのか、相手に語ってあげると喜ぶかもしれない。
報酬：【視聴率】10%増加、700【ソウル】',
               '『親友の復讐』
キャラクターから一人を選び、相手が「小道具」で修得している特技を指定特技として判定を行う。判定に成功すると、相手が身につけていた装身具が、あなたの親友を殺害した犯人の特徴とぴったり合致することがわかる。相手への関係を、属性「犯人」で獲得する。【深度】は1になる。
クリア条件：相手への関係の【深度】を2以上にする。すると、相手は自白する。そのあとに戦闘を挑み、一度相手を倒せ。死んだものは帰ってこないが、あなたは満足する。
報酬：【視聴率】10%増加、400【ソウル】',
               '『護身』
キャラクターから一人を選び、相手が「願望」で修得している特技を指定特技として判定を行う。判定に成功すると、相手はかつてあなたに断罪された事があり、それを恨みに思っていることが明らかになる。相手への関係を、属性「復讐者」で獲得する。【深度】は１になる。
クリア条件：結果フェイズまで標的に殺されない。すると、なんとかやりすごせる･･････が、次はどうなることやら。
報酬：【視聴率】10%増加、相手が回収人なら500【ソウル】相手がＮＰＣなら100【ソウル】',
               '『心酔』
回収人から一人を選び、相手が「動作」で修得している特技を指定特技として判定を行う。判定に成功すると、相手を第一印象から善人だと確信し、いろんな理屈をつけて相手の願いごとを正義だと信じ込む。相手への関係を、属性「善人」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を2以上にする。すると、相手の願いごとの本質を理解し、酔いから醒めたり、忠実な協力者となることを約束したりする。
報酬：【視聴率】10%増加、200【ソウル】',
               '『悪党』
キャラクターから一人を選び、相手が「職業」で修得している特技を指定特技として判定を行う。判定に成功すると、相手はその筋では有名な悪い職業人であることが判明する。こいつを放っておけば悲しむ人が出るだろう。相手への関係を、属性「悪人」で獲得する。【深度】は１になる。
クリア条件：悪人を許せないなら、相手を戦闘で一度倒す。または、改心させたいなら、相手への関係の【深度】を3にする。相手が本当に改心するかどうかは関係なく、会心の説得を撃ち放ったことで、あなたは満足する。
報酬：【視聴率】10%増加、クリア条件達成時の相手への関係【深度】�200【ソウル】',
               '『サイドキック』
回収人から一人を選び、相手が「衣装」で修得している特技を指定特技として判定を行う。判定に成功すると、相手は自分の相棒として悪を倒すのに最適な人物であると確信する。サイドキックだ。相手への関係を、属性「仲間」で獲得する。【深度】は１になる。
クリア条件：自分が登場しているシーンで、相手が誰かを倒し、そのあと自分が殺されずに戦闘が終了する。相手を褒めて、仲間にふさわしい人物だと宣言してあげよう。
報酬：【視聴率】10%増加、300【ソウル】',
              ]
    when /^TSPT$/i, /^TrainingSPT$/i
      tableName = "サブプロット表「修行」"
      table = [
               '『戦闘狂』
回収人から一人を選び、相手が「動作」で修得している特技を指定特技として判定を行う。判定に成功すると、周りにいる強そうな人に対して対抗心が沸いてくる。結局、勝つのは一人なんだから、今から叩いておいたほうがいい。相手への関係を、属性「敵」で獲得する。【深度】は1になる。
クリア条件：相手を含む三人のキャラクターを一回以上死亡させる。すると、あなたの戦闘熱はおさまる。そこから仲良くできるかどうかは微妙なところかもしれない。
報酬：【視聴率】10%増加、600【ソウル】',
               '『弟子入り』
キャラクターから一人を選び、相手が「職業」で修得している特技を指定特技として判定を行う。判定に成功すると、あなたは、その職業の世界的権威を発見する。故あって引退して久しいが、教えを請えば今後の人生の役に立つだろう。教える気があるかどうかは知らないが。相手への関係を、属性「師匠」で獲得する。【深度】は1になる。
クリア条件：相手への関係の【深度】を3にする。すると、相手は何かを教えてくれる。あるいは、もう教えることはなにもないと言うだろう。
報酬：【視聴率】10%増加、相手が回収人なら400【ソウル】相手がNPCなら600【ソウル】',
               '『ライバル』
キャラクターから一人を選び、相手が「願望」で修得している特技を指定特技として判定を行う。成功すると、相手はあなたに特定の分野で勝利したことのある人物だとわかる。奴に勝たねば、あなたは先に進めない。相手への関係を、属性「ライバル」で獲得する。【深度】は1になる。
クリア条件：2サイクル目以降で相手を一度倒す。あなたが倒すまで、相手は他のキャラクターに倒されてはならない。お前を倒すのは俺だって決まってるんだからな！
報酬：【視聴率】10%増加、相手が回収人なら300【ソウル】相手がNPCなら500【ソウル】',
               '『改造』
回収人から一人を選び、相手が「衣装」で修得している特技を指定特技として判定を行う。判定に成功すると、相手に比べて自分の肉体が貧弱すぎるような気がしてくる。改造してもっと強くならなければ。相手への関係を、属性「基準線」で獲得する。【深度】は1になる。
クリア条件：相手よりも多くのヘルサイバーをインストールした状態で結果フェイズを迎える。この時数えるのは個数だけ、そして同じ数ではダメだ。
報酬：【視聴率】10%増加、600【ソウル】',
               '『装備改善』
回収人から一人を選び、相手が「小道具」で修得している特技を指定特技として判定を行う。判定に成功すると、相手の持っている装備について興味が湧く。自分の能力上昇のヒントになるかもしれない。相手への関係を、属性「いい装備」で獲得する。【深度】は1になる。
クリア条件：相手への関係の【深度】を2以上にする。すると、相手は持ち物について解説してくれるか、あるいはキッパリと断る。あなたの探索はそこで終わるが、経験は残る。
報酬：【視聴率】10%増加、200【ソウル】',
               '『特訓』
回収人から一人を選び、相手が「情動」で修得している特技を指定特技として判定を行う。判定に成功すると相手は自分と同じく修行不足を痛感しているのだと確信する。一緒にトレーニングだ。相手への関係を、属性「特訓仲間」で獲得する。【深度】は1になる。
クリア条件：相手への関係の【深度】を2以上にする。すると、あなたと相手はなんらかの手応えを感じて、特訓は終わる。このタイミングでキラートリックの名前を変えたりしてもかまわない。
報酬：【視聴率】10%増加、200【ソウル】',
              ]
    when /^BSPT$/i, /^BeamSPT$/i
      tableName = "サブプロット表「笑い」"
      table = [
               '『大サービス』
キャラクターから一人を選び、相手が「情動」で修得している特技を指定特技として判定を行う。判定に成功すると、この殺伐とした番組を和やかにできるのは、自分と相手しかいないと確信する。相手への関係を、属性「相棒」で獲得する。【深度】は１になる。
クリア条件：相手と一緒に。「サービス」シーンの判定に２回成功する。すると、なんだか番組が楽しくなったような気もするが……変わってない気もする。あなたの戦いはまだ続く。
報酬：【視聴率】10%増加、あなたと相手の両方に300【ソウル】',
               '『脱げ』
キャラクターから一人を選び、相手が「衣装」で修得している特技を指定特技として判定を行う。判定に成功すると、相手の衣装をいい感じで脱がせればウケが取れると確信してしまう。相手への関係を、属性「脱がせたい」で獲得する。【深度】は１になる。
クリア条件：相手に戦闘を挑み。キラートリックで倒す。このとき、相手の衣装はバラバラに粉砕され、ヘルスタイリストが相手を罵倒しながら新しいものを用意することになる。
報酬：【視聴率】10%増加、500【ソウル】',
               '『いい話』
キャラクターから一人を選び、相手が「動作」で修得している特技を指定特技として判定を行う。判定に成功すると、相手はヘコんでいるのか嫌なことがあるのか、表情に乏しい。笑顔を取り戻してやらなければ。相手への関係を、属性「暗い奴」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を３にする。相手は少し笑顔を見せたかもしれないし、光の加減でそう見えただけかもしれない。だが、あなたは確かな手応えを感じる。
報酬：【視聴率】10%増加、相手が回収人なら400【ソウル】相手がNPCなら600【ソウル】',
               '『笑い屋』
キャラクターから一人を選び、相手が「願望」で修得している特技を指定特技として判定を行う。判定に成功すると、相手は援護がないとサービスシーンがキツそうだな、と直感する。相手への関係を、属性「真面目な奴」で獲得する。【深度】は１になる。
クリア条件：相手の登場する「サービス」シーンに自分も登場し、カメラの外から笑い声を出すロールプレイを行う。
報酬：【視聴率】10%増加、あなたと相手の両方に100【ソウル】',
               '『戦闘回避』
キャラクターから一人を選び、相手が「職業」で修得している特技を指定特技として判定を行う。判定に成功すると、あなたは相手の願いごとに深く共感してしまう。そのため、相手の邪魔はしたくはなくなる。相手への関係を、属性「勝つべき人」で獲得する。【深度】は１になる。
クリア条件：結果フェイズまで直接相手を殺さない。すると、誰も気づかないかもしれないが、あなたは自分のやるべきことをやったことになる。
報酬：【視聴率】10%増加、200【ソウル】',
               '『親友』
キャラクターから一人を選び、相手が「小道具」で修得している特技を指定特技として判定を行う。判定に成功すると、その小道具についての会話をきっかけに、相手への興味が芽生える。もしかしたら友達になれるかもしれない。相手への関係を、属性「友人」で獲得する。【深度】は１になる。
クリア条件：相手への関係の【深度】を２以上にする。すると、あなたと相手は友情のようなもので結ばれるかもしれないし、やはり無理だったのだと、あなたが諦めるかもしれない。
報酬：【視聴率】10%増加、200【ソウル】',
              ]
      
    end
    
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end
  
  
  def getCmTableResult()
    tableName = "CM表"
    table = [
             [11, 'あなたのＴシャツ／決めポーズの姿が背中にプリントされたＴシャツです。そして、フロントにはあの名台詞が！'],
             [12, 'あなたのアクションフィギュア／放送中から大人気！職人が一つ一つ入魂の手作り！夜になると、込められた魂が泣き叫びます。'],
             [13, 'ヘルーレイボックス／シーズン６６６の全てがここに……特典映像満載で送る、豪華ボックスセット！予約特典はヘルくんストラップです。'],
             [14, 'ヘルエアコン／これさえあれば、火事の最中でも気温は氷点下！逆に家を燃やすこともできますよ。温度調節機能は有料オプションです！'],
             [15, 'ヘルおむつ／ヘルギャザーで漏れをストップ、６６６年間履きっぱなしでも平気！中身がどうなるか保証しません、ヘルおむつ'],
             [16, 'ヘルサプリ／寝なくても平気！カラフルな幻覚が見える！でもメイン機能は健康増進です。ヘルサプリで健康生活を！'],
             [22, 'ヘルツアーズ／お得な料金で地獄百景を回ろう！苦しむ人間との魂の触れ合いが満載！ヘルツアーズでは、他社ではできない体験をお約束します。'],
             [23, 'ヘルテレビ／夜中に女性が出てきますが、気にしなければ大丈夫！アナログヘルテレビは政府の協賛を得ています。'],
             [24, 'ヘルビール／アルコール度数６６６％、ヘルホップ使用。地獄の渇きにはヘルビール……違いの分かる人のために。'],
             [25, 'ヘルフォン／ドヤ顔のノマド気取りをぶちのめすこともできます。そう、ヘルフォンならね。'],
             [26, 'ヘルまんじゅう／数千年のご愛顧をいただいております、地獄銘菓ヘルまんじゅう。うまい、ヘルうますぎる。しかしながら、食べると死にます。'],
             [33, 'ヘルモービル／人を轢けば轢くほど速くなり……走りは別次元へ。ヘルモービルが、お求めやすい価格で新登場。'],
             [34, 'ヘル会話教室／隣人をうまく罵れない？イヤな奴に言いくるめられた？ヘル会話を学べば、どんな相手も激昂させられます！'],
             [35, 'ヘル金融／金利６６６％の安心がここに。ヘル金融の取り立ては、非常に紳士的だと評判です。'],
             [36, 'ヘル汁(健康飲料)／辺獄で採れた新鮮な毒草を、そのまま粉末にしました。これを飲めば、もう病気や老化の心配はいりません。健康飲料ヘル汁。'],
             [44, 'ヘル整形外科／無保険かつ無免許ですが、どんな注文にも応えます。ヘル整形外科は、あなたのおっしゃる通りの外見を実現します！'],
             [45, 'ヘル生命保険／すぐ死ぬあなたに、ヘル生命の安全コース。残された人には地獄が待っています。'],
             [46, 'ヘル洗剤／どんなにしつこい血糊や汚物も一撃！どんな色の服も地獄の黒に染まります！ヘル洗剤、好評発売中！'],
             [55, 'ヘル洗濯機／６６６層式、音速を超えた回転があなたの衣装を粉々に！なにをしたかったのか忘れれば壮観です。ヘル洗濯機！'],
             [56, 'ヘル掃除機／好評の壁面粉砕機能に加えて、今年のモデルには屋根吸引機能も！ヘル掃除機のスイッチを入れるだけで、あなたの家がなくなります。'],
             [66, 'ヘル入浴剤／体が溶ける！新感覚で話題を呼んだヘル入浴剤がついに登場！死体の処理には使わないでください。'],
            ]
    result, number = get_table_by_d66_swap(table)
    
    return tableName, result, number
  end
  
  
  def getErTableResult()
    tableName = "蘇生副作用表"
    table = [
             [11, 'おいそそうな匂いがするようになった'],
             [12, 'おまけでヤギが出てきた'],
             [13, '髪が蛍光色に変わった'],
             [14, '声がロボっぽくなった'],
             [15, '先が三角形になったシッポが生えた'],
             [16, '頭上に謎のプログレスバーが出ている'],
             [22, '性別が変わった'],
             [23, '全身にモザイクがかかった'],
             [24, '全体的に半透明になった'],
             [25, 'ちょっと腐った'],
             [26, '肌が蛍光色に変わった'],
             [33, '羽が生えた'],
             [34, '番組参加中の記憶を失った'],
             [35, '額に第三の目が開いた'],
             [36, '微妙に浮いてる'],
             [44, 'ひよこにかわった'],
             [45, '不合格品のスタンプを押された'],
             [46, '太った'],
             [55, '耳が好きな動物のものに変わった'],
             [56, '痩せた'],
             [66, '立派な角が額に生えた'],
            ]
    result, number = get_table_by_d66_swap(table)
    
    return tableName, result, number
  end
  
  def getWKTableResult() 
    tableName = "一週間表"
    table = [
             [11, '家を追い出されたヘルＰが泊まりに来た'],
             [12, '危険なオーラを醸し出せるようになり、非常にモテた'],
             [13, '地獄落ちを覚悟し、身辺整理を行った'],
             [14, '地獄テレビ情報雑誌にインタビューを受けた'],
             [15, '実家の商売が好調になるが、客がみんなヘルピープル'],
             [16, '信号待ちをしていたら、仕事中の悪魔にサインを求められた'],
             [22, '親戚が番組を見ていて、電話で感想を伝えてきた'],
             [23, '次の週の護衛天使と知り合いになった'],
             [24, '次の週の標的と知り合いになった'],
             [25, 'トラウマが残って全く眠れず、酒に溺れた'],
             [26, '願いが叶うなら働かなくていいし、仕事を辞めた'],
             [33, '願いとは別に出演報酬が振り込まれたので豪遊した'],
             [34, '番組中のノリで人をはたいたら、なんだか死んでしまった'],
             [35, '番組や悪魔が怖くなって世界中を逃げ回った'],
             [36, 'ファンを名乗るストーカーに自宅を荒らされた'],
             [44, 'ふと思い立って山に籠り、修行に明け暮れた'],
             [45, 'ヘルスタイリストに自宅の衣類をすべて燃やされた'],
             [46, 'ヘルドクターが借金の申し込みに来た'],
             [55, 'ヘルドレスの偽物を売りつけようとする下級悪魔と戦った'],
             [56, '他の回収人のうち一人と同じ職場だったことが明らかに'],
             [66, '妙に寄付を募る営業電話がかかってくる週だった'],
            ]
    result, number = get_table_by_d66_swap(table)
    
    return tableName, result, number
  end

  
  def getServiceSceneTableResult()
    tableName = "サービスシーン表"
    sceneTable = [
                  '脱衣系サービスシーン表',
                  '暴力系サービスシーン表',
                  '旅行系サービスシーン表',
                  '恋愛系サービスシーン表',
                  '感動系サービスシーン表',
                  '別ジャンルサービスシーン表',
                 ]
    sceneGroup, number1 = get_table_by_1d6(sceneTable)
    
    case sceneGroup
    when "脱衣系サービスシーン表"
      table = [
               '衣装の不調',
               '着替え',
               '脱衣麻雀',
               '入浴',
               '水着で戦う',
               '野球拳',
              ]
    when "暴力系サービスシーン表"
      table = [
               '勝ったあとの暴力',
               '集団の暴力',
               '暴力をのほめかす会話',
               '負けた後の暴力',
               'まったく無意味な暴力',
               '無意味な暴力',
              ]
    when "旅行系サービスシーン表"
      table = [
               '移動機関の紹介',
               '地元の遊びの紹介',
               '地元の動物の紹介',
               '地元の宿の紹介',
               '寝起きドッキリ',
               '名物料理の紹介',
              ]
    when "恋愛系サービスシーン表"
      table = [
               'キスシーン',
               '恋に気付く',
               '恋に悩む',
               'デートシーン',
               '秘密を告白する',
               'ラブシーン',
              ]
    when "感動系サービスシーン表"
      table = [
               '家族の愛に気付く',
               'かっこいいことを言う',
               '失敗して落ち込む',
               '地元住民と触れ合う',
               '死んだ人の話をする',
               '高いところでかっこいいことをいう',
              ]
    else
      table = [
               'サイコサスペンス',
               'スペースオペラ',
               '童話',
               'ヒロイックファンタジー',
               'モンスターホラー',
               'ロボ',
              ]
    end
    
    resultTmp, number2 = get_table_by_1d6(table)
    result = "#{sceneGroup}「#{resultTmp}」を行う。"
    number = "#{number1}#{number2}"
    
    return tableName, result, number
  end
  
  
  def getSoulTableResult()
    tableName = "ソウル放出表"
    table = [
             '100',
             '200',
             '200',
             '200',
             '300',
             '300',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end
  
  
  def getSceneTelopGenericTableResult()
    tableName = "汎用演出表"
    table = [
             '大きくて分かりやすい文字でテロップが出る',
             '回想に合わせて再現ビデオが流れる',
             '関係者が登場人物について語る',
             'ゲストヘルピープルが行動を論評する',
             '行動の是非についてヘル視聴者の投票が始まる',
             'ヘルバンドの新曲がBGMとして流れる',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end
  
  
  def getHairStylistAbuseTableResult(type)
    tableName = "ヘルスタイリスト罵倒表"
    
    hellStylistAbuseTable1 = [
                              [11, ' 汁まみれになった'],
                              [12, ' お爺ちゃんがこのまえ捨てた'],
                              [13, ' 失敗メイクの'],
                              [14, ' 低予算の映画のモンスター代わりにされた'],
                              [15, ' 仲間はずれの'],
                              [16, ' 悪い意味で成分無調整の'],
                              [22, ' 宇宙人が中身を抜いていったあとの'],
                              [23, ' 化学処理班が処理しにきそうな'],
                              [24, ' 革命直後にギロチンにかけられた'],
                              [25, ' 狂気の芸術家が想像で作りあげた'],
                              [26, ' 故郷の村を帝国軍に焼かれた直後の'],
                              [33, ' 五年ぶりにベッドの下から出てきた'],
                              [34, ' 高速道路でなんども轢かれた'],
                              [35, ' 死後六週間の'],
                              [36, ' 失敗料理を無理に食べた'],
                              [44, ' 小さい子供がはじめて描いた'],
                              [45, ' 箪笥と壁の隙間に住み着いた'],
                              [46, ' 猫がくわえてもってきた'],
                              [55, ' 有名なタコっぽいモンスターを見てしまった'],
                              [56, ' 旅行先で財布がないことに気づいた'],
                              [66, ' 猟奇殺人を趣味にしている'],
                             ]
    hellStylistAbuseTable2 = [
                              [11, ' アルマジロ'],
                              [12, ' イソギンチャク'],
                              [13, ' カメムシ'],
                              [14, ' カモノハシ'],
                              [15, ' クジャク'],
                              [16, ' クラゲ'],
                              [22, ' ゴキブリ'],
                              [23, ' ゴミムシ'],
                              [24, ' サル'],
                              [25, ' ドブネズミ'],
                              [26, ' ナマケモノ'],
                              [33, ' ナマコ'],
                              [34, ' うんこ'],
                              [35, ' フンコロガシ'],
                              [36, ' ペンギン'],
                              [44, ' ミジンコ'],
                              [45, ' 古い人形'],
                              [46, ' 人食い熊'],
                              [55, ' 毒トカゲ'],
                              [56, ' 毛虫'],
                              [66, ' 野良犬'],
                             ]
                             
      hellStylistwtable1 = [
               '「まるで『',
               '「まるで『',
               '「まさに『',
               '「まさに『',
               '「あなたは『',
               '「あなたは『',
              ]
      hellStylistwtable2 = [
               '』みたいよ！」',
               '』みたいだわ！」',
               '』じゃないの！」',
               '』ね。」',
               '』に見えるわ。」',
               '』そっくりよ！」',
              ]
    
    case type
      when 1
        result, number = get_table_by_d66_swap(hellStylistAbuseTable1)
      when 2
        result, number = get_table_by_d66_swap(hellStylistAbuseTable2)
      else
        result1, num1 = get_table_by_d66_swap(hellStylistAbuseTable1)
        result2, num2 = get_table_by_d66_swap(hellStylistAbuseTable2)
        before,  = get_table_by_1d6(hellStylistwtable1)
        after, = get_table_by_1d6(hellStylistwtable2)
        result = "#{before}#{result1}#{result2}#{after}"
        number = "#{num1},#{num2}"
    end
    
    return tableName, result, number
  end
  
  
  def getSkillTableResult(type)
    skillTableFull = [
                      ['職業', ['無職', '芸術家', '研究者', '家事手伝い', '学生', '悪漢', '労働者', '探偵', '大物', '医師', '公務員']],
                      ['動作', ['叫ぶ', '閃く', '斬る', '振る', '投げる', '殴る', '蹴る', '跳ぶ', '撃つ', '掴む', '待つ']],
                      ['小道具', ['ピアス', '髪飾り', '銃', 'ネックレス', 'ベルト', '眼鏡', '帽子', '時計', '剣', 'リング', 'タトゥー']],
                      ['衣装', ['ネイキッド', 'アウトドア', 'エスニック', 'ヒップホップ', 'ミリタリー', 'フォーマル', 'トラッド', 'ゴシック', 'パンク', 'メタル', 'アイドル']],
                      ['情動', ['愛', '喜び', '期待', '焦り', '自負', '怒り', '悲しみ', '嫉妬', '恐怖', '恥', '嫌悪']],
                      ['願望', ['死', '復讐', '勝利', '支配', '獲得', '繁栄', '強化', '安全', '健康', '長寿','生']],
                     ]
    skillTable, num1 = get_table_by_1d6(skillTableFull)
    skillGroup, table = skillTable
    if(type == "T")
      tableName = "指定特技ランダム決定表"
      skill, num2 = get_table_by_2d6(table)
      result = "「#{skillGroup}」《#{skill}》"
      number = "#{num1},#{num2}"
    else
      tableName = "指定特技分野ランダム決定表"
      result = skillGroup
      number = num1
    end
    
    
    return tableName, result, number
  end
  
  def getExtraTableResult(type)
    tableName = "エキストラ表"
    extraTable1 = [
				   [11, "あなたの親友である"],
				   [12, "あなたと恨んで付け狙っている"],
				   [13, "いそがしく電話で話し込んでいる"],
				   [14, "ころんで逃げ損ねた"],
				   [15, "シーンの背景の持ち主である"],
				   [16, "架空の人物だと思われていた神"],
				   [22, "過去のシーズン優勝者である"],
				   [23, "気分よく酔っ払った"],
				   [24, "恐怖で身をすくませている"],
				   [25, "業界では有名な"],
				   [26, "幸せな家庭を持つ"],
				   [33, "広域指名手配されている"],
				   [34, "今朝のニュースで特集されていた"],
				   [35, "常に微笑みを絶やさない大物"],
				   [36, "真実の探求の過程で発狂した"],
				   [44, "地元では負けを知らない"],
				   [45, "非常に動作にキレのある"],
				   [46, "普通の"],
				   [55, "変身ヒーローの正体である"],
				   [56, "『#{getNameTableResult(0)[1]}』の異名を持つ"],
				   [66, "歴史上の人物だが実は生きていた"],
	              ]
	extraTable2 = [
				   [11, "サラリーマン"],
				   [12, "スポーツ選手"],
				   [13, "チンピラ"],
				   [14, "ドライバー"],
				   [15, "ねずみ"],
				   [16, "パイロット"],
				   [22, "映画監督"],
				   [23, "犬"],
				   [24, "刺客"],
				   [25, "主婦"],
				   [26, "小説家"],
				   [33, "雀士"],
				   [34, "政治家"],
				   [35, "大金持ち"],
				   [36, "大男"],
				   [44, "謎の美女"],
				   [45, "猫"],
				   [46, "美少女"],
				   [55, "文化人"],
				   [56, "勇者"],
				   [66, "神"],
	              ]
    extraTable3 = [
				   [11, "怪しい箱の中から"],
				   [12, "哀れな犠牲者を殺しながら"],
				   [13, "牛に乗りつつ"],
				   [14, "馬に乗りつつ"],
				   [15, "壁を粉砕しながら"],
				   [16, "壁を粉砕しながら"],
				   [22, "濃い霧の中からゆっくりと"],
				   [23, "自動ドアを抜けながら"],
				   [24, "上空から急降下しつつ"],
				   [25, "全速力で走りつつ"],
				   [26, "高いところから"],
				   [33, "テーマ音楽とともに"],
				   [34, "通りがかりに"],
				   [35, "名前と職業のテロップと一緒に"],
				   [36, "バイクに乗りつつ"],
				   [44, "墓をやぶって"],
				   [45, "部下を大勢引き連れて"],
				   [46, "吹きすさぶ風を纏い"],
				   [55, "武器を構えつつ"],
				   [56, "ヘルポータルを通って"],
				   [66, "炎をバックに"],
	              ]
	extraTable4 = [
				   [11, "愛らしく登場"],
				   [12, "あざやかに登場"],
				   [13, "あっさりと登場"],
				   [14, "怪しく登場"],
				   [15, "荒々しく登場"],
				   [16, "勢い良く登場"],
				   [22, "美しく登場"],
				   [23, "偉そうに登場"],
				   [24, "おごそかに登場"],
				   [25, "恐ろしく登場"],
				   [26, "かっこよく登場"],
				   [33, "気取って登場"],
				   [34, "死の予感とともに登場"],
				   [35, "しめやかに登場"],
				   [36, "上品に登場"],
				   [44, "だらしなく登場"],
				   [45, "知的に登場"],
				   [46, "なごやかに登場"],
				   [55, "なめらかに登場"],
				   [56, "不機嫌に登場"],
				   [66, "陽気に登場"],
	              ]
	
    case type
    when 1
      result, number = get_table_by_d66_swap(extraTable1)
    when 2
      result, number = get_table_by_d66_swap(extraTable2)
    when 3
      result, number = get_table_by_d66_swap(extraTable3)
    when 4
      result, number = get_table_by_d66_swap(extraTable4)
    else
      result1, num1 = get_table_by_d66_swap(extraTable1)
      result2, num2 = get_table_by_d66_swap(extraTable2)
      result3, num3 = get_table_by_d66_swap(extraTable3)
      result4, num4 = get_table_by_d66_swap(extraTable4)
      result = "#{result1}#{result2}が#{result3}#{result4}"
      number = "#{num1},#{num2},#{num3},#{num4}"
    end
    
    return tableName, result, number
  end

  def getpcTableResult()
    tableName = "製作委員決定表"
    table = [
             [11, 'ヘルライオン'],
             [12, 'ヘルライオン'],
             [13, 'ヘルライオン'],
             [14, 'ヘルクロウ'],
             [15, 'ヘルクロウ'],
             [16, 'ヘルクロウ'],
             [22, 'ヘルスネーク'],
             [23, 'ヘルスネーク'],
             [24, 'ヘルスネーク'],
             [25, 'ヘルドラゴン'],
             [26, 'ヘルドラゴン'],
             [33, 'ヘルドラゴン'],
             [34, 'ヘルフライ'],
             [35, 'ヘルフライ'],
             [36, 'ヘルフライ'],
             [44, 'ヘルゴート'],
             [45, 'ヘルゴート'],
             [46, 'ヘルゴート'],
             [55, 'ヘルベア'],
             [56, 'ヘルベア'],
             [66, 'ヘルベア'],
            ]
    result, number = get_table_by_d66_swap(table)
    
    return tableName, result, number
  end

  def getohTableResult()
    tableName = "実際どうだったのか表"
    table = [
             'おおむね【興行収入】の予測は正確だった。増減はなしで、回収人はしばらく公共の場所でモテる。',
             '思ったよりいけた！【興行収入】が4D6億ヘル円増加。ジャンルの佳作として五年後も話に上る。',
             'ちょっとゴタついて公開時期を逃した。【興行収入】が4D6億ヘル円減少。ヘルーレイの売り上げに期待しよう。',
             '何の間違いか、同時期に他シーズンの映画版が公開される。【興行収入】は半分になる。',
             'なんの間違いか、ヘルアカデミー賞をいっぱい取った。【興行収入】は2倍になる！',
             'エピソード中の表現が微妙かつ厄介な問題を引き起こし、たくさんのヘルピープルが嫌な気分になったうえ公開ナシに。',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end

  def getplTableResult()
    tableName = "ヘルライオンタスク表"
    table = [
             '「フランス語。」指定特技は《芸術家》。なぜか登場人物全員がフランス語でしか会話できないシーンが設定される。知っている限りのフランス語を駆使して意思疎通を試みろ！',
             '「会話がない。」指定特技は《跳ぶ》。会話そのものが下劣でエンターテイメント的だということで、全く言葉を発することが許されないシーン表が設定される。ジェスチャーで頑張れ！',
             '「名言の引用。」指定特技は《髪飾り》。過去の名作映画や小説、歴史上の出来事などからパクってきた名言しか口にできないシーンが設定される。教養がないと喋れない！',
             '「登場人物の増加。」指定特技は《トラッド》。登場人物が無駄に倍に増え、それぞれ個性的な悩みや事件を語り始める。このままでは面倒だしギャラもかさむので、全員殺そう。',
             '「漠然とした不安。」指定特技は《自負》。特定のものごとではなく、ぼんやりとした不安感に延々苦しんだり落ち込んだりするシーンを強要される。これを退屈にしないのは難しい！',
             '「長回し。」指定特技は《生》。カットの切り替わりなしで、30分ほど同じシーンが続く。NGは許されない。これで間を持たせられるのはプロの俳優くらいのものだが、回収人は演技の素人だ。',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end
  def getprTableResult()
    tableName = "ヘルクロウタスク表"
    table = [
             '「しょぼいゲスト。」指定特技は《悪漢》。一般人や犬、弟やお母さんがゲストとして登場する。本来のゲストとの差額は着服される。事情を知らされていないゲストとうまく絡め！',
             '「実家での撮影。」指定特技は《叫ぶ》。撮影場所が実家になる。本来のロケ費用は着服される。家人を説得して、うまく撮影許可をもらわなければ赤字が出てしまうだろう！',
             '「普段着での撮影。」指定特技は《眼鏡》。なぜかヘルスタイリストが休みで、その分の人件費と衣装代は着服される。普段着のまま超常のアクションをこなすのは不可能に近い！',
             '「徒歩で移動する。」指定特技は《ゴシック》。ヘルポータルの予算を着服された。ひたすらシーンまで歩かないと撮影すらできない。撮影前に疲れ切ってしまうと、観客にも伝わるだろう。',
             '「特殊効果が出ない。」指定特技は《期待》。リアルタイム特殊効果の予算を着服された！予算なしで働こうとしないヘルSFXエンジニアを、なんとか泣き落としで働かせろ！',
             '「食事が出ない。」指定特技は《獲得》。食事の時間だが、ヘル仕出し弁当が来ない。その分の予算を着服されたのだ。空腹のままいい演技をするには、気合に頼るしかない。',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end
  def getpnTableResult()
    tableName = "ヘルスネークタスク表"
    table = [
             '「仇。」指定特技は《家事手伝い》。過去改変により、任意の相手が、急に親の仇や恋人の仇になる。戸惑わずに怒りに燃えよう。なぜこうなったのか、考えてしまってはダメだ！',
             '「些細なことから大喧嘩。」指定特技は《待つ》。任意の相手と、どうでもいい理由でけんかをすることを強要される。本当にどうでもよくなって仲たがいをやめることは許されない。',
             '「利益が相反する。」指定特技は《ネックレス》。任意の相手の願いごとと自分の願いごとが両立しないことをアピールさせられる。そうでなくても、むりやり理由をひねり出せ！',
             '「神敵。」指定特技は《ネイキッド》。趣味や宗教を理由に、任意の相手と仲たがいすることを強要される。話してわかってしまったり、趣味の幅が広がってはいけない。',
             '「幼児期のトラウマ。」指定特技は《嫉妬》。過去改変で哀しい記憶が捏造され、猟奇殺人犯としてキャラ付される。若干引いてしまったり、悲しさを乗り越えてはダメだ。狂え！',
             '「人類一般への憎しみ。」指定特技は《復讐》。理由なく人を憎むことを強要される。もともと性格のいい回収人にとってはかなりの難行だ。人類の悪いところ探しをしよう！',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end
  def getpdTableResult()
    tableName = "ヘルドラゴンタスク表"
    table = [
             '「バリエーションを見たくてNG。」指定特技は《大物》。別に悪い演技ではないのに、ちょっと違うバリエーションを何度もやらされるぞ。キレずに耐えてくれ！',
             '「なんか違うからNG。」指定特技は《殴る》。回収人の演技に難癖をつけられた。しかしどんな演技をすればいいのか教えてはくれない。キレずに耐えてくれ！他の人を殴れ！',
             '「まだ時間があるからNG。」指定特技は《タトゥー》。クソみたいな理由でNGを出されたが、とにかくキレずに耐えてくれ。ヘルドラゴンが機嫌を損ねると広告費が削られる！',
             '「予算を使い切りたくてNG。」指定特技は《ネイキッド》。まだ人件費が使えるからもう一回、とかいう理由でやり直しだ。キレずに耐えてくれ。ヤツは金を持ってるんだ！',
             '「機嫌が悪いのでNG。」指定特技は《怒り》。家で何かイヤなことがあったので難癖をつけられた。もう、なにをしてもOKは出ない。キレずに耐えてくれ。いつかは終わる！',
             '「お前そのものがNG。」指定特技は《安全》。回収人の人選そのものに難癖をつけられた。あなたにNGを出すことだけが目的だ。キレずに耐えてくれ。相手は超強いのだ。',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end
  def getpfTableResult()
    tableName = "ヘルフライタスク表"
    table = [
             '「汚いトイレ。」指定特技は《探偵》。すごく不衛生な場所に背景が変更される。肥溜めと見まごうばかりのトイレか、肥溜めだ。臭いに耐えられるかどうか、そこが勝負の分かれ目だ！',
             '「芸術的陰部隠し。」指定特技は《跳ぶ》。全裸でシーンを過ごすが映ってはいけないものがある。本来なら水着の布地がある部分がカメラに映らないように、障害物を活用しろ！',
             '「触手。」指定特技は《ベルト》。唐突に触手モンスターが登場する。彼は紳士的だが容赦なく襲いかかってくるので、セクシーに負けろ！さもなくば観客はがっかりするだろう！',
             '「ヌルヌル。」指定特技は《トラッド》。なんだか粘液が入ったバケツが登場する。これを被ってシーンを始めろというのだ。成分もわからない粘液はイヤだが、なんとか耐えろ！',
             '「病原体を経口摂取する。」指定特技は《恐怖》。病気の原因になりそうなものを無理やり食べることに。具体的には排泄物や腐敗したもの、最初から不衛生なものなどだ。',
             '「局部へのひどい打撃。」指定特技は《死》。ヘルADや他のキャラクターから局部を強打されることを強要される。死なずにうまく痛がれ。下手をするとショックでそのまま死ぬ！',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end
  def getpgTableResult()
    tableName = "ヘルゴートタスク表"
    table = [
             '「アイドルが仲間として登場。」指定特技は《医師》。推しアイドルが回収人の味方として登場するので、スムーズに関係を築いて共闘し、相手を立てよう。リスペクトだ！',
             '「アイドルが敵として登場。」指定特技は《蹴る》。推しアイドルが標的の味方。回収人の敵として登場！見せ場を用意してあげつつ、うまく倒せ。一方的ではダメだ！',
             '「アイドルが無意味に登場。」指定特技は《剣》。推しアイドルが登場するが、なぜ登場するのか誰もわからない。本人も役を教えられていない。即興で何とかしろ！',
             '「アイドルの仲間になる。」指定特技は《エスニック》。推しアイドルの仲間となり、唐突に恋や冒険をすることに。黄金の魂と関係ない話が続くが、観客にそれを気づかせるな！',
             '「アイドルと握手。」指定特技は《愛》。推しアイドルが急に出てきて握手してくれる。急なことだが、とまどってはいけない。流れるように話題を作って会話し、握手へのコメントをしよう」！',
             '「混ざってダンス。」指定特技は《健康》。いきなりダンスシーンが始まる！ただし回収人はバックダンサーだ。歌え！踊りきれ！息切れしたり笑顔が引きつるとNGだ！',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end
  def getpbTableResult()
    tableName = "ヘルベアタスク表"
    table = [
             '「ヘルふとん。」指定特技は《大物》。地獄のような寝心地を提供する、一枚200万ヘル円のヘルふとんをアピールすることに。素材は絶対そんなにしないから、技術を強調しよう。',
             '「ヘルハンドバッグ。」指定特技は《掴む》。無限にものが入るハンドバッグをアピールだ。ただし狙ったものを取り出せないし、バッグのなかにバッグを入れると、ひどいことが起きる。',
             '「へルマスク。」指定特技は《帽子》。完全にヘルPの頭部を再現した高級マスクを被り、その素晴らしさをアピールしながら映画を撮影しよう。なお、視界はすごく悪い。',
             '「ヘルカメラ。」指定特技は《アイドル》。かならず心霊写真が写るカメラを観客に売りつけろ！霊障が起きることもあるんだが、それを悟られてはいけない。写真をはっきり見られるな！',
             '「ヘルウォッチ。」指定特技は《焦り》。時間はわからないが超かっこいい時計をアピール。それってブレスレットじゃないのか、と突っ込まれる前に逃げ切るんだ！',
             '「ヘルお札。」指定特技は《繁栄》。願いを成就する悪霊が、あなたの成功を確約！その後は成功したまま事故で死ぬことになるが、そんなの些細なことだ！',
            ]
    result, number = get_table_by_1d6(table)
    
    return tableName, result, number
  end

end

