#--*-coding:utf-8-*--

class MonotoneMusium < DiceBot
  
  def initialize
    super
    
    @sendMode = 2
    @d66Type = 1
    @sortType = 1
  end
  def gameName
    'モノトーン・ミュージアム'
  end
  
  def gameType
    "MonotoneMusium"
  end
  
  def prefixs
    ['2D6.*', 'ET','ET2', 'OT', 'DT', 'DT2', 'WDT']
  end
  
  def getHelpMessage
    return <<INFO_MESSAGE_TEXT
・判定
　・通常判定　　　　　　2D6+m>=t[c,f]
　　修正値m,目標値t,クリティカル値c,ファンブル値fで判定ロールを行います。
　　クリティカル値、ファンブル値は省略可能です。([]ごと省略できます)
　　自動成功、自動失敗、成功、失敗を自動表示します。
・各種表
　・感情表　ET／感情表 2.0　ET2
　・兆候表　　OT
　・歪み表　DT／歪み表ver2.0　DT2
　・世界歪曲表　　WDT
・D66ダイスあり
INFO_MESSAGE_TEXT
  end
  
  
  def rollDiceCommand(command)
    
    result = checkRoll(command)
    return result unless(result.empty?)
    
    debug("判定ロールではなかった")
    
    debug("各種表として処理")
    return rollTableCommand(command)
  end
  
  
  def checkRoll(string)
    output = ''
    
    crit = 12
    fumble = 2
    
    return output unless(/^2D6([\+\-\d]*)>=(\d+)(\[(\d+)?(,(\d+))?\])?$/i =~ string)
    
    modText = $1
    target = $2.to_i
    crit = $4.to_i if($4)
    fumble = $6.to_i if($6)
    
    mod = 0
    mod = parren_killer("(0#{modText})") unless( modText.nil? )
    
    total, dice_str, = roll(2, 6, @sortType && 1)
    total_n = total + mod.to_i
    
    output = "#{total}[#{dice_str}]＋#{mod} → #{total_n}"
    
    if(total >= crit)
      output += " ＞ 自動成功"
    elsif(total <= fumble)
      output += " ＞ 自動失敗"
    elsif(total_n >= target)
      output += " ＞ 成功"
    else
      output += " ＞ 失敗"
    end
    
    output = "(#{string}) ＞ #{output}"
    
    return output
    
  end
  
  
  def rollTableCommand(command)
    output = ''
    type = ""
    
    case command
    when /ET2/i
      type ="感情表2.0"
      output, total_n = mm_emotion_table_ver2()
    when /ET/i
      type ="感情表"
      output, total_n = mm_emotion_table()
    when /OT/i
      type ="兆候表"
      output, total_n = mm_omens_table()
    when /DT2/i
      type = "歪み表ver2.0"
      output, total_n = mm_distortion_table_ver2()
    when /WDT/i
      type = "世界歪曲表"
      output, total_n = mm_world_distortion_table()
    when /DT/i
      type = "歪み表"
      output, total_n = mm_distortion_table()
    end
    
    output = "#{type}(#{total_n}) ＞ #{output}" if(output != '')
    
    return output
  end

  # 感情表(d66)[ET]
  def mm_emotion_table
    table = [
      "【信頼（しんらい）】",
      "【有為（ゆうい）】",
      "【友情（ゆうじょう）】",
      "【純愛（じゅんあい）】",
      "【慈愛（じあい）】",
      "【憧れ（あこがれ）】",
      "【恐怖（きょうふ）】",
      "【脅威（きょうい）】",
      "【憎悪（ぞうお）】",
      "【不快感（ふかいかん）】",
      "【食傷（しょくしょう）】",
      "【嫌悪（けんお）】",
      "【好意（こうい）】",
      "【庇護（ひご）】",
      "【遺志（いし）】",
      "【懐旧（かいきゅう）】",
      "【尽力（じんりょく）】",
      "【忠誠（ちゅうせい）】",
      "【不安（ふあん）】",
      "【侮蔑（ぶべつ）】",
      "【嫉妬（しっと）】",
      "【劣等感（れっとうかん）】",
      "【優越感（ゆうえつかん）】",
      "【憐憫（れんびん）】",
      "【尊敬（そんけい）】",
      "【感服（かんぷく）】",
      "【慕情（ぼじょう）】",
      "【同情（どうじょう）】",
      "【傾倒（けいとう）】",
      "【好奇心（こうきしん）】",
      "【偏愛（へんあい）】",
      "【執着（しゅうちゃく）】",
      "【悔悟（かいご）】",
      "【警戒心（けいかいしん）】",
      "【敵愾心（てきがいしん）】",
      "【忘却（ぼうきゃく）】",
    ]
    return get_table_by_d66(table)
  end

  # 感情表2.0(d66)[ET2]
  def mm_emotion_table_ver2
    table = [
      "【プレイヤーの任意】",
      "【同一視（どういつし）】",
      "【連帯感（れんたいかん）】",
      "【幸福感（こうふくかん）】",
      "【親近感（しんきんかん）】",
      "【誠意（せいい）】",
      "【懐旧（かいきゅう）】",
      "【同郷（どうきょう）】",
      "【同志（どうし）】",
      "【くされ縁（くされえん）】",
      "【期待（きたい）】",
      "【好敵手（こうてきしゅ）】",
      "【借り（かり）】",
      "【貸し（かし）】",
      "【献身（けんしん）】",
      "【義兄弟（ぎきょうだい）】",
      "【幼子（おさなご）】",
      "【親愛（しんあい）】",
      "【疎外感（そがいかん）】",
      "【恥辱（ちじょく）】",
      "【憐憫（れんびん）】",
      "【隔意（かくい）】",
      "【嫌悪（けんお）】",
      "【猜疑心（さいぎしん）】",
      "【厭気（けんき）】",
      "【不信感（ふしんかん）】",
      "【怨念（おんねん）】",
      "【悲哀（ひあい）】",
      "【悪意（あくい）】",
      "【殺意（さつい）】",
      "【敗北感（はいぼくかん）】",
      "【徒労感（とろうかん）】",
      "【黒い泥（くろいどろ）】",
      "【憤懣（ふんまん）】",
      "【無関心（むかんしん）】",
      "【プレイヤーの任意】",
    ]
    return get_table_by_d66(table)
  end

 # 兆候表(2d6)[OT]
  def mm_omens_table
    table = [
      "【信念の喪失】\n[出自]を喪失する。特徴は失われない。",
      "【昏倒】\nあなたは[戦闘不能]になる。",
      "【肉体の崩壊】\nあなたは 2D6点のHPを失う。",
      "【放心】\nあなたはバッドステータスの[放心]を受ける。",
      "【重圧】\nあなたはバッドステータスの[重圧]を受ける。",
      "【現在の喪失】\n現在持っているパートナーをひとつ喪失する。",
      "【マヒ】\nあなたはバッドステータスの[マヒ]を受ける。",
      "【邪毒】\nあなたはバッドステータスの[邪毒]5を受ける。",
      "【色彩の喪失】\n漆黒、墨白、透明化……。その禍々しい色彩の喪失は他らなぬ異形化の片鱗だ。",
      "【理由の喪失】\n[境遇]を喪失する。特徴は失われない。",
      "【存在の喪失】\nあなたの存在は一瞬、この世界から消失する。",
    ]
    
    return get_table_by_2d6(table)
  end

  # 歪み表(2D6)[DT]
  def mm_distortion_table
    table = [
      "【世界消失】\n演目の舞台がすべて失われる。舞台に残っているのはキミたちと異形、伽藍だけだ。クライマックスフェイズへ。",
      "【生命減少】\n演目の舞台となっている街や国から動物や人間の姿が少なくなる。特に子供の姿は見られない。",
      "【空間消失】\n演目の舞台の一部（建物一棟程度）が消失する。",
      "【天候悪化】\n激しい雷雨に見舞われる。",
      "【生命繁茂】\nシーン内に植物が爆発的に増加し、建物はイバラのトゲと蔓草に埋没する。",
      "【色彩喪失】\n世界から色彩が失われる。紡ぎ手（PC）以外の人々は世界のすべてをモノクロームになったかのように認識する。",
      "【神権音楽】\n美しいが不安を覚える音が流れる。音は人々にストレスを与え、街の雰囲気は悪化している。",
      "【鏡面世界】\n演目の舞台に存在するあらゆる文字は鏡文字になる。",
      "【時空歪曲】\n昼夜が逆転する。昼間であれば夜になり、夜であれば朝となる。",
      "【存在修正】\nGMが任意に決定したNPCの性別や年齢、外見が変化する。",
      "【人体消失】\nシーンプレイヤーのパートナーとなっているNPCが消失する。どのNPCが消失するかは、GMが決定する。",
    ]
    return get_table_by_2d6(table)
  end
 
  # 歪み表ver2.0(d66)[DT2]
  def mm_distortion_table_ver2
    table = [
      "【色彩侵食】\nシーン内に存在するあらゆる無生物と生物は、白と黒とのモノトーンの存在となる。紡ぎ手は【縫製】難易度8の判定に成功すれば、この影響を受けない。この効果は歪みをもたらした異形の死によって解除される。",
      "【色彩侵食】\nシーン内に存在するあらゆる無生物と生物は、白と黒とのモノトーンの存在となる。紡ぎ手は【縫製】難易度8の判定に成功すれば、この影響を受けない。この効果は歪みをもたらした異形の死によって解除される。",
      "【虚無現出】\nほつれの中から虚無がしみ出す。シーンに登場しているエキストラは虫一匹にいたるまで消滅し、二度と現れない。",
      "【虚無現出】\nほつれの中から虚無がしみ出す。シーンに登場しているエキストラは虫一匹にいたるまで消滅し、二度と現れない。",
      "【季節変容】\n季節が突如として変化する。1D6し、1なら春、2なら夏、3なら秋、4なら冬、5ならプレイしている現在の季節、6ならGMの任意とせよ。",
      "【季節変容】\n季節が突如として変化する。1D6し、1なら春、2なら夏、3なら秋、4なら冬、5ならプレイしている現在の季節、6ならGMの任意とせよ。",
      "【ほつれ】\n世界がひび割れ、ほつれが現出する。ほつれに触れたものは虚無に飲み込まれ、帰ってくることはない。",
      "【ほつれ】\n世界がひび割れ、ほつれが現出する。ほつれに触れたものは虚無に飲み込まれ、帰ってくることはない。",
      "【異形化】\nシーン内のすべてのエキストラは何らかの異形化を受ける。これを治癒する術はない。異形の群れ（『インカルツァンド』P.237）×1D6と戦闘させてもよい。",
      "【異形化】\nシーン内のすべてのエキストラは何らかの異形化を受ける。これを治癒する術はない。異形の群れ（『インカルツァンド』P.237）×1D6と戦闘させてもよい。",
      "【死の行進】\n人々の心に虚無が広がり、不安と絶望に満たされていく。逃れられぬ恐怖から逃れようと、人々はみずから、あるいは無意識のうちに死へと向かい行動を始める。",
      "【死の行進】\n人々の心に虚無が広がり、不安と絶望に満たされていく。逃れられぬ恐怖から逃れようと、人々はみずから、あるいは無意識のうちに死へと向かい行動を始める。",
      "【時間加速】\nシーン内に存在するあらゆる無生物と生物は、2D6年ぶん時間が加速する。生物なら老化する。紡ぎ手は【縫製】難易度8の判定に成功すれば、この影響を受けない。",
      "【時間加速】\nシーン内に存在するあらゆる無生物と生物は、2D6年ぶん時間が加速する。生物なら老化する。紡ぎ手は【縫製】難易度8の判定に成功すれば、この影響を受けない。",
      "【時間逆流】\nシーン内に存在するあらゆる無生物と生物は、2D6年ぶん時間が逆流する。生物ならば若返る。製造年／生年より前に戻った場合は虚無に飲まれ消滅する。紡ぎ手は【縫製】難易度8の判定に成功すれば、この影響を受けない。",
      "【時間逆流】\nシーン内に存在するあらゆる無生物と生物は、2D6年ぶん時間が逆流する。生物ならば若返る。製造年／生年より前に戻った場合は虚無に飲まれ消滅する。紡ぎ手は【縫製】難易度8の判定に成功すれば、この影響を受けない。",
      "【災害到来】\n嵐、火山の噴火、洪水など、ほつれによって乱された自然が、人々に牙をむく。",
      "【災害到来】\n嵐、火山の噴火、洪水など、ほつれによって乱された自然が、人々に牙をむく。",
      "【人心荒廃】\n歪みによってもたらされた不安と恐怖は、人々を捨て鉢にさせる。",
      "【人心荒廃】\n歪みによってもたらされた不安と恐怖は、人々を捨て鉢にさせる。",
      "【平穏無事】\n何も起きない。紡ぎ手は背筋が凍るほどの恐怖を覚える。",
      "【平穏無事】\n何も起きない。紡ぎ手は背筋が凍るほどの恐怖を覚える。",
      "【疫病蔓延】\n登場しているキャラクターは【肉体】難易度8の判定を行ない、失敗すると［邪毒］5を受ける。病の治療法の有無などについてはGMが決定する。迷ったら、伽藍を倒すと病も消滅する、とせよ。",
      "【疫病蔓延】\n登場しているキャラクターは【肉体】難易度8の判定を行ない、失敗すると［邪毒］5を受ける。病の治療法の有無などについてはGMが決定する。迷ったら、伽藍を倒すと病も消滅する、とせよ。",
      "【異端審問】\n異端審問の時は近い。PCたちが紡ぎ手であることが知れ渡れば、PCたちも火刑台へと送られることになろう。",
      "【異端審問】\n異端審問の時は近い。PCたちが紡ぎ手であることが知れ渡れば、PCたちも火刑台へと送られることになろう。",
      "【歪み出現】\nほつれの破片から歪みが現れ、人々を襲撃する。病魔（『インカルツァンド』P.238）×2と戦闘を行なわせてもよい。また、別の異形でもよい。",
      "【歪み出現】\nほつれの破片から歪みが現れ、人々を襲撃する。病魔（『インカルツァンド』P.238）×2と戦闘を行なわせてもよい。また、別の異形でもよい。",
      "【悪夢現出】\nシーン内のあらゆる人々は恐るべき恐怖の夢を見る。心弱きものたちは、伽藍にすがるか、異端者を火あぶりにせねばこの夢から逃れられぬと考えるだろう。",
      "【悪夢現出】\nシーン内のあらゆる人々は恐るべき恐怖の夢を見る。心弱きものたちは、伽藍にすがるか、異端者を火あぶりにせねばこの夢から逃れられぬと考えるだろう。",
      "【鼠の宴】\n鼠の大群が出現し、穀物を食い荒らし、疫病をまき散らす。大鼠（『MM』P.240）×1D6と戦闘を行なわせてもよい。",
      "【鼠の宴】\n鼠の大群が出現し、穀物を食い荒らし、疫病をまき散らす。大鼠（『MM』P.240）×1D6と戦闘を行なわせてもよい。",
      "【歪曲御標】\n歪んだ御標が下される。歪み表（『MM』P.263）を振ること。",
      "【歪曲御標】\n歪んだ御標が下される。歪み表（『MM』P.263）を振ること。",
      "【地域消滅】\n演目の舞台となっている地域そのものが消えてなくなる。影響下にあるすべてのキャラクター（伽藍を含む）は【縫製】難易度10の判定に成功すれば脱出できる。失敗した場合即座に死亡する。エキストラは無条件で死亡する。",
      "【地域消滅】\n演目の舞台となっている地域そのものが消えてなくなる。影響下にあるすべてのキャラクター（伽藍を含む）は【縫製】難易度10の判定に成功すれば脱出できる。失敗した場合即座に死亡する。エキストラは無条件で死亡する。",
    ]
    return get_table_by_d66(table)
  end

  # 世界歪曲表(2D6)[WDT]
  def mm_world_distortion_table
    table = [
      "【消失】\n世界からボスキャラクターが消去され、消滅する。エンディングフェイズへ。",
      "【自己犠牲】\nチャートを振ったPCのパートナーとなっているNPCのひとりが死亡する。チャートを振ったPCのHPとMPを完全に回復させる。",
      "【生命誕生】\nキミたちは大地の代わりに何かの生き物の臓腑の上に立っている。登場しているキャラクター全員に［邪毒］5を与える。",
      "【歪曲拡大】\nシーンに登場している紡ぎ手ではないNPCひとりが漆黒の凶獣（『MM』P.240）に変身する。",
      "【暴走】\n“ほつれ ”がいくつも生まれ、シーンに登場しているすべてのキャラクターの剥離値を +1 する。",
      "【幻像世界】\n周囲の空間は歪み、破壊的なエネルギーが充満する。次に行なわれるダメージロールに+5D6する。",
      "【変調】\n右は左に、赤は青に、上は下に、歪みが身体の動きを妨げる。登場しているキャラクター全員に［狼狽］を与える。",
      "【空間消失】\n演目の舞台が煙のように消失する。圧倒的な喪失感により、登場しているキャラクター全員に［放心］を与える。",
      "【生命消失】\n次のシーン以降、エキストラは一切登場できない。現在のシーンのエキストラに関してはGMが決定する。",
      "【自己死】\nもっとも剥離値の高いPCひとりが［戦闘不能］になる。複数のPCが該当した場合はGMがランダムに決定する。",
      "【世界死】\n世界の破滅。難易度12の【縫製】判定に成功すると破滅から逃れられる。失敗すると行方不明になる。エンディングフェイズへ。",
    ]
    
    return get_table_by_2d6(table)
  end


end
